# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/09_filt.ipynb.

# %% auto 0
__all__ = ['makefilt', 'filt', 'drop', 'dropnils', 'just', 'allbut', 'justabc', 'justnum', 'justpunc', 'droppunc', 'dropquote',
           'splitget', 'prepart', 'midpart', 'endpart']

# %% ../nbs/09_filt.ipynb 4
import string
from typing import Union, Callable, Optional, TypeAlias

from atyp import StrQ, StrGuard
from nchr import U1, DOT, NIL, SPACE
from chck.strs import isquote
from ltyp import notnilstr

# %% ../nbs/09_filt.ipynb 6
def makefilt(sflag: Union[str, StrGuard]) -> StrGuard:
    'Make a filter from a boolean function.'
    if isinstance(sflag, str):
        def chrflag(s: str) -> bool: return s in sflag
    elif isinstance(sflag, Callable):
        def chrflag(s: str) -> bool: return sflag(s)
    else:
        def chrflag(s: str) -> bool: return s == sflag
    return chrflag

# %% ../nbs/09_filt.ipynb 7
def filt(*args: str, flag: StrGuard) -> tuple:
    return list(filter(makefilt(flag), args))
    
def drop(*args: str, flag: StrGuard) -> tuple:
    flag = makefilt(flag)
    call = lambda c: not flag(c)
    return filt(*args, flag=call)

# def dropnils(*args: str) -> tuple:
#     return drop(*args, flag=NIL)

def dropnils(*args, drop: bool = True) -> tuple:
    return tuple(filter(notnilstr, args)) if drop else args

# %% ../nbs/09_filt.ipynb 8
def just(s: str, flag: StrGuard) -> str:
    '''Filters a string for just the specified characters'''
    return NIL.join(list(filter(makefilt(flag), s)))

def allbut(s: str, flag: StrGuard) -> str:
    '''Filters a string for all but the specified characters'''
    flag = makefilt(flag)
    call = lambda c: not flag(c)
    return just(s, call)

def justabc(s: str) -> str:
    '''Filters a string for just alpha values'''
    return just(s, str.isalpha)

def justnum(s: str) -> str:
    '''Filters a string for just numeric values'''
    return just(s, str.isnumeric)

def justpunc(s: str) -> str:
    '''Filters a string for just punctuation values'''
    return just(s, string.punctuation)
    
def droppunc(s: str) -> str:
    return s.translate(str.maketrans(NIL, NIL, string.punctuation))

def dropquote(s: str) -> str:
    return just(s, lambda c: not isquote(c))

def splitget(s: str, sep: str = U1, idx: int = 0) -> str:
    parts = s.split(sep)
    return parts[idx]

def prepart(s: str, sep: StrQ = SPACE) -> str:
    pre, *_ = s.partition(sep)
    return pre

def midpart(s: str, sep: StrQ = SPACE) -> str:
    _, mid, _ = s.partition(sep)
    return mid

def endpart(s: str, sep: StrQ = SPACE) -> str:
    *_, end = s.partition(sep)
    return end
